﻿using System;
using System.Collections.Specialized;
using System.Linq;
using System.Web;

using Sitecore;
using Sitecore.Configuration;
using Sitecore.Data;
using Sitecore.Data.Fields;
using Sitecore.Data.Items;
using Sitecore.Resources.Media;
using Sitecore.SecurityModel;
using Sitecore.Shell.Framework.Commands;
using Sitecore.Text;
using Sitecore.Web.UI.Sheer;

namespace Kevin
{
    public class Thumbnail : Sitecore.Shell.Framework.Commands.Command
    {
        /// <summary>
        /// Determine if the command button should be displayed or hidden.
        /// </summary>
        public override CommandState QueryState(CommandContext context)
        {
            if (context.Items != null && context.Items[0] != null && context.Items[0].Template != null)
            {
                Item contextItem = context.Items[0];

                // Only display the button if the context item is a Template, an XSL Rendering, or a Sublayout.
                if ( contextItem.Template.Key == "template" ||
                     contextItem.Template.Key == "xsl rendering" ||
                     contextItem.Template.Key == "sublayout" )
                {
                    return CommandState.Enabled;
                }
            }

            // Hide the button for all other items.
            return CommandState.Hidden;
        }

        /// <summary>
        /// Execute the command (the button has been clicked)
        /// </summary>
        public override void Execute(CommandContext context)
        {
            if (context.Items.Length > 0)
            {
                // Get the current Sitecore item.
                Sitecore.Data.Items.Item currentItem = context.Items[0];
                if (currentItem != null)
                {
                    // Create a parameter list.
                    NameValueCollection parameters = new NameValueCollection();

                    // Pass in the current Sitecore item's ID.
                    parameters["currentid"] = currentItem.ID.ToString();

                    // If the current item already has a value in it's thumbnail field, pass that in as well.
                    Sitecore.Data.Fields.ImageField thumbnailField = currentItem.Fields["__Thumbnail"];
                    if (thumbnailField != null)
                        parameters["selectedid"] = thumbnailField.MediaID.ToString();

                    // Start the pipeline.
                    Context.ClientPage.Start(this, "Run", parameters);
                }
            }
        }

        /// <summary>
        /// The pipeline method - gets called above and when the media browser dialog is closed.
        /// </summary>
        protected void Run(ClientPipelineArgs args)
        {
            Database masterDb = Factory.GetDatabase("master");

            // A postback indicates that the media browser dialog was closed.
            if (args.IsPostBack)
            {
                // If the user cancelled, we do nothing.
                if ( string.IsNullOrEmpty(args.Result) || args.Result == "undefined" ) return;

                using (new SecurityDisabler())
                {
                    // args.Result contains the ID of the image that was selected in the media browser.
                    Sitecore.Data.Items.MediaItem selectedImageItem = masterDb.GetItem(new ID(args.Result));
                    if (selectedImageItem != null)
                    {
                        // Get the current item so that we can edit it.
                        Sitecore.Data.Items.Item item = masterDb.GetItem(args.Parameters["currentid"]);

                        // Get the __Thumbnail field from the item.
                        Sitecore.Data.Fields.ImageField thumbnailField = item.Fields["__Thumbnail"];
                        if (thumbnailField != null)
                        {
                            // Begin editing.
                            item.Editing.BeginEdit();

                            // Some Google searching indicated this was the best way to update an ImageField.
                            thumbnailField.Alt = selectedImageItem.Alt;
                            thumbnailField.MediaID = selectedImageItem.ID;
                            thumbnailField.Src = MediaManager.GetMediaUrl(selectedImageItem);
                            thumbnailField.MediaPath = selectedImageItem.MediaPath;
                            thumbnailField.SetAttribute("showineditor", "1");

                            // Finish editing (save changes).
                            item.Editing.EndEdit();
                        }
                    }
                }
            }
            else
            {
                // If this is not a postback, we want to display Sitecore's media browser dialog.
                UrlString url = new UrlString(UIUtil.GetUri("control:Sitecore.Shell.Applications.Media.MediaBrowser"));

                // We want the browser to be rooted in the Images folder in the media library.
                Item folderItem = masterDb.GetItem("/sitecore/media library/Images");
                url["ro"] = folderItem.Uri.ToString();

                // If there is an existing thumbnail for the current item, we want the media browser to pre-select it.
                if (!string.IsNullOrEmpty(args.Parameters["selectedid"]))
                {
                    Sitecore.Data.Items.Item selectedImageItem = masterDb.GetItem(new ID(args.Parameters["selectedid"]));
                    if (selectedImageItem != null)
                        url["fo"] = selectedImageItem.Uri.ToString();
                }

                // These two lines displays the dialog and causes a postback to this method when it is closed.
                SheerResponse.ShowModalDialog(url.ToString(), true);
                args.WaitForPostBack(true);
            }
        }
    }
}
